#
# calc.tcl
#

#
# calculate rmse value only 
#
proc calc_rmse { } {
	global panvar xcv vPlot

	set pid 0 
	set pNum $vPlot($pid,pNumCur)
	set dNum $vPlot($pid,$pNum,dNumCur)

	# get parameter list 
	set parmList ""
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		lappend parmList "=$panvar(parms,$np)"
	}

	# call fastcrvfit
	u_log " -> Calc RMSE button"
	write_crvfit_data $pid $pNum $dNum $xcv(fn,dataTmp)
	if { [call_fastcrvfit $parmList] == "FALSE" } {
		return 
	}

	# update parameter fields and variables
	set f [open $xcv(fn,resShortTmp)]
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		gets $f line
	}
	gets $f line 
	set nd [scan $line "%f" panvar(rmse)]
	gets $f line 
	set nd [scan $line "%d" panvar(itNum)]
	close $f

	set line  "Output - rmse=$panvar(rmse)\n"
	u_log $line

	# update results text file 
	get_output_fname
	exec mv -f $xcv(fn,resLongTmp) $panvar(fn,resSave)

	# update the screen
	calc_fit_one $pid $pNum $vPlot($pid,$pNum,dNumCur)
	do_plot $pid 0 
}

#
# perform calculations trying to converge parameter set. 
#
proc calc_iterate { } {
	global myData panvar xcv xwin vPlot

	set pid 0 
	set pNum $vPlot($pid,pNumCur)
	set dNum $vPlot($pid,$pNum,dNumCur)

	# get parameter list 
	set parmList ""
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		if {[string compare $panvar(choice,$np) "yes"] == 0} {
			lappend parmList "=$panvar(parms,$np)"
		} else {
			lappend parmList $panvar(parms,$np)
		}
	}

	# call fastcrvfit
	u_log " -> Best Fit button"
	write_crvfit_data $pid $pNum $dNum $xcv(fn,dataTmp)
	exec rm -f $xcv(fn,resShortTmp) $xcv(fn,resLongTmp)
	if { [call_fastcrvfit $parmList] == "FALSE" } {
		u_error "Unable to converge to a better\n solution. Try changing the
		parameters or setting\n more parameters as constants."
		return 
	}

	# update parameter fields and variables
	set f [open $xcv(fn,resShortTmp)]
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		if {[gets $f line] >= 0} {
			set nd [scan $line "%f %f" num std]
			set panvar(newParms,$np) $num 
			set panvar(parms,$np) $num
			set panvar(stddev,$np) $std
		}
	}
	gets $f line
	set nd [scan $line "%f" panvar(rmse)]
	gets $f line
	set nd [scan $line "%d" panvar(itNum)] 
	close $f

	# update results text file 
	get_output_fname
	exec mv -f $xcv(fn,resLongTmp) $panvar(fn,resSave)

	# make log file entry
	set num [format "%s" $myData(0,$panvar(dataIndex),iDataName)]
	set line "  Best Fit Output - DataSet Label: $num" 
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		set num [format "%#7.5g %#-7.4g" $panvar(parms,$np) $panvar(stddev,$np)]
		set label [lindex [$xwin(parms).p$np.label configure -text] 4]
		set line [concat $line $label: $num]
	}
	set line  "$line rmse=$panvar(rmse) iterations=$panvar(itNum)\n" 
	u_log $line

	# update the screen
	calc_fit_one $pid $pNum $vPlot($pid,$pNum,dNumCur)
	do_plot $pid 0
}

#
# call fast crvfit c program 
#
proc call_fastcrvfit { parmList } {
	global panvar xcv

	set cmd "$xcv(exe,FASTCRV) $xcv(fn,dataTmp) \
		$xcv(fn,resLongTmp) $xcv(fn,resShortTmp) $panvar(functionKey) \
		$panvar(maxIt) $parmList"

	u_log "Calling fastcrvfit: $cmd"

	# execute fastcrvfit function
	if [catch {eval exec $cmd} err] {
		u_log "Unable to converge, Error message is '$err'"
		return FALSE 
	}
	return TRUE
}

#####################################################################
#
# Write current crvfit data into a file 
#
proc write_crvfit_data { pid pNum dNum fn } {
	global vPlot myData panvar 

# make sure file can be opened
	if [catch {open $fn w} fileId] {
		u_error "Cannot open crvfit data file '$fn'."
	}
	set mix $vPlot($pid,$pNum,$dNum,mix)

	if {$myData(0,iSetNum) > 1} {
		puts $fileId "# $myData(0,$mix,iDataName)"
	}

	set count 0
	for {set i 0} {$i < $myData(0,$mix,iDataNum)} {incr i} {
		if [info exists vPlot($pid,$pNum,bounded_left)] {
			set xVal [bounded_convert 0 $mix $i]
			if { $xVal < $vPlot($pid,$pNum,bounded_left) } { continue }
			if { $xVal > $vPlot($pid,$pNum,bounded_right) } { continue }
		}
		if {$myData(0,$mix,nCol) == 2} {
			puts $fileId "$myData(0,$mix,$i,xd) $myData(0,$mix,$i,yd)"
		} else {
			puts $fileId "$myData(0,$mix,$i,xd) $myData(0,$mix,$i,yd) $myData(0,$mix,$i,td)"
		}
		incr count
	}
	close $fileId
	if { $count < 2 } {
		u_error "Dataset $mix has less than 2 points"
	}
}

#####################################################################
#
# Create output filename based on datafile and dataset
#
proc get_output_fname {} {
    global xcv panvar

	set fn [file tail $panvar(data,filename)]
	set fit $panvar(functionKey)
	set panvar(fn,output) "out.$fit.$panvar(dataLabel)"
	set panvar(fn,resSave) "$xcv(MY_HOME)/$panvar(fn,output).txt"
}
