
#####################################################################
#
# xcrvfit gridfit window 
#
#####################################################################

#####################################################################
#
# Create main panel for gridfit
#
proc do_gridfit { } {
	global xcv myData vPlot panvar

	set pid 1 
	set iFile 0
	set mix 0

	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		set panvar(gfit,choice,$np) $panvar(choice,$np) 
	}

	# show gridfit-select plot with first dataset
	plot_newdata $pid $mix $iFile
	set myData($iFile,$mix,gridFlag) "TRUE"
	make_gridfit_panel $pid

# set some parameters for testing
#	set panvar(gfit,lb,0) 1.0
#	set panvar(gfit,ub,0) 10 
#	set panvar(gfit,inc,0) 1.0
#	set panvar(gfit,choice,0) yes 
#	set panvar(gfit,choice,1) no
#	set panvar(gfit,choice,2) yes 
#	set panvar(gfit,choice,3) yes 
#	set panvar(gfit,choice,4) yes 

}

#####################################################################
#
# Create main panel for gridfit
#
proc make_gridfit_panel { pid } {
	global xcv vPlot panvar myData myFunc

	set panvar(gfit,colWidth) [expr 3 + [string length $myData(0,0,iDataName)]]

	set f ".$xcv(win,gridfit,main)"
	set tab $f.table
	set databar $f.databar
	set parm $f.parm

	toplevel $f

	frame $databar
	pack $databar -pady 10
	label $databar.label -text "DataSets: "
	button $databar.all -text "Select All" \
		-command "bind_select_all $pid $tab TRUE"
	button $databar.none -text "Select None" \
		-command "bind_select_all $pid $tab FALSE"
	pack $databar.label $databar.all $databar.none -anchor w -side left

	frame $tab
	pack $tab
	set rowNum 0
	set colNum [expr $vPlot($pid,col_max) + 1] 
	for {set ix 0 } {$ix < $myData(0,iSetNum)} {incr ix} {
		if {$colNum > $vPlot($pid,col_max)} {
			incr rowNum
			frame $tab.$rowNum -padx 20
			pack $tab.$rowNum -anchor w
			set colNum 1
		}
		set panvar(gfit,$ix,row) $rowNum
		set panvar(gfit,$ix,col) $colNum
		set resBut $tab.$rowNum.$colNum
		label $resBut -text $myData(0,$ix,iDataName) \
			-relief raised -pady 4 -width $panvar(gfit,colWidth) 
		$tab.$rowNum.$colNum configure -background $vPlot($pid,no_color) 
		bind $resBut <Button-1> "bind_select_data $pid $tab.$rowNum.$colNum $ix"
		if { $myData(0,$ix,gridFlag) == "FALSE" } {
			$resBut configure -background $vPlot($pid,no_color) 
		} else {
			$resBut configure -background $vPlot($pid,yes_color) 
		}
		pack $resBut -side left
		incr colNum
	}

	frame $parm
	pack $parm -pady 10
	frame $parm.title
	pack $parm.title -padx 10
	label $parm.title.1 -text "Parameter" -width 14
	label $parm.title.2 -text "Lower_Bound" -width 12
	label $parm.title.3 -text "Upper_Bound" -width 12
	label $parm.title.4 -text "Increment" -width 12
	label $parm.title.5 -text "Hold_Constant" -width 18
	pack $parm.title.1 $parm.title.2 $parm.title.3 $parm.title.4 \
		$parm.title.5 -ipadx 0 -padx 0 -side left

	for {set np 0} {$np < $panvar(parms,num)} {incr np} {

		frame $parm.p$np
		pack $parm.p$np -padx 10

		set pLab [lindex $myFunc($panvar(functionKey),parms) $np]
		label $parm.p$np.item -text $pLab -width 8

		command_entry $parm.p$np.start "" 0 "bind_parm $np" \
			-textvar panvar(newParms,$np) -width 12 
		command_entry $parm.p$np.ub "" 0 "" \
			-textvar panvar(gfit,ub,$np) -width 12 
		command_entry $parm.p$np.inc "" 0 "" \
			-textvar panvar(gfit,inc,$np) -width 12 
		$parm.p$np.start.entry configure -background $vPlot($pid,no_color) 
		$parm.p$np.ub.entry configure -background $vPlot($pid,no_color) 
		$parm.p$np.inc.entry configure -background $vPlot($pid,no_color) 

		set cf [frame $parm.p$np.choice -borderwidth 0]
		radiobutton $cf.0 -variable panvar(gfit,choice,$np) \
			-text "yes" -value "yes" -command "bind_hold_constant $parm $np"
		pack $cf.0 -side left 
		radiobutton $cf.1 -variable panvar(gfit,choice,$np) \
			-text "no" -value "no" -command "bind_hold_constant $parm $np"
		pack $cf.1 -side left

		pack $parm.p$np.item $parm.p$np.start $parm.p$np.ub $parm.p$np.inc \
			-side left
		pack $cf -padx 5 -side left
		bind_hold_constant $parm $np
	}

	frame $f.but
	pack $f.but -pady 10 -padx 20
	button $f.but.start -text "Go" -command "gridfit_calc"
	button $f.but.quit -text "Cancel" -command "destroy $f"
	pack $f.but.start $f.but.quit -side left 
}

#####################################################################
#
# User clicks on one of the datasets 
#
proc bind_select_data { pid item iSet } {
	global myData panvar vPlot xcv

	set pid 1

	# update panel
	if { $myData(0,$iSet,gridFlag) == "FALSE" } {
		set myData(0,$iSet,gridFlag) "TRUE" 
		$item configure -background $vPlot($pid,yes_color) 
	} else {
		set myData(0,$iSet,gridFlag) "FALSE" 
		$item configure -background $vPlot($pid,no_color) 
	}

	# update our plot parameters 
	set current_data -1
	set count 0
	set pNum 0
	for {set mix 0 } {$mix < $myData(0,iSetNum)} {incr mix} {
		if { $myData(0,$mix,gridFlag) == "TRUE" } {
			plot_set_data $pid $pid $pNum $count 0 $mix
			# hilite a dataset when it is added to the list 
			if { $mix == $iSet } {
				set vPlot($pid,0,$count,data_color) $vPlot($pid,data_color)
				set current_data $count 
			} else {
				set vPlot($pid,0,$count,data_color) $vPlot($pid,data_color_opt)
				set vPlot($pid,0,$count,data_label_text) "" 
			}
			incr count
		}
	}
	set vPlot($pid,0,dNumTot) $count
	update

	# draw what we have
	do_plot $pid 0

	# draw current data last and in a different color so that it is easy to see 
	if { $current_data >= 0 } { 
		set vPlot($pid,0,$current_data,data_color) $vPlot($pid,data_color)
		draw_data $pid 0 $current_data 0
	}
}

#####################################################################
#
# Select all or none of the datasets for grid fitting
# 
proc bind_select_all { pid tab val } {
	global myData panvar vPlot xcv

	set pid 1

	# update panel
	for {set ix 0 } {$ix < $myData(0,iSetNum)} {incr ix} {
		set myData(0,$ix,gridFlag) $val
		set rowNum $panvar(gfit,$ix,row)
		set colNum $panvar(gfit,$ix,col)
		if {$val == "TRUE"} {
			$tab.$rowNum.$colNum configure -background $vPlot($pid,yes_color) 
		} else {
			$tab.$rowNum.$colNum configure -background $vPlot($pid,no_color) 
		}
	}

	# update our plot parameters 
	set count 0
	set pNum 0
	for {set mix 0 } {$mix < $myData(0,iSetNum)} {incr mix} {
		if { $myData(0,$mix,gridFlag) == "TRUE" } {
			plot_set_data $pid $pid $pNum $count 0 $mix
			set vPlot($pid,0,$count,data_color) $vPlot($pid,data_color_opt)
			set vPlot($pid,0,$count,data_label_text) "" 
			incr count
		}
	}
	set vPlot($pid,0,dNumTot) $count
	update

	# draw what we have
	do_plot $pid 0
}

#####################################################################
#
# User hits the Go button
#
proc gridfit_calc { } {
	global myFunc panvar myData vPlot xcv

	set f ".$xcv(win,gridfit,calc)"

	u_log " -> Gridfit Button"

	# Get copy of parameter state
	set panvar(parmGrid) "" 
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		if [info exists panvar(parms,$np)] {
			set panvar(parmsCopy,$np) $panvar(parms,$np) 
		}
	}

	set dataFlag 0
	for {set ix 0 } {$ix < $myData(0,iSetNum)} {incr ix} {
		if { $myData(0,$ix,gridFlag) == "TRUE" } {
			set dataFlag 1
			break
		}
	}
	if { $dataFlag == 0 } {
		u_error "No dataSets selected"
	}

	# Show output panel if we have selected data
	show_gridfit_results_panel $f

	# start with current value of shown parameters
	set gridParmList ""
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		if [info exists panvar(newParms,$np) ] {
			lappend gridParmList $panvar(newParms,$np) 
		} else {
			lappend gridParmList "?"
		}
	}

	# process each dataset
	set pid 3
	$f.results.text insert 3.0 "\n********************\n\n"
	set pNum 0
	for {set ix 0 } {$ix < $myData(0,iSetNum)} {incr ix} {
		if { $myData(0,$ix,gridFlag) == "TRUE" } {
			set vPlot($pid,$pNum,0,mix) $ix
			u_log "DataSet: $myData(0,$ix,iDataName)"
			$f.live.dataset configure -textvariable panvar(dataLabel)
			$f.live.rmseVal configure -textvariable panvar(gfit,$pNum,outrmse)
			$f.live.rmseLow configure -textvariable panvar(gfit,$pNum,rmseLow)
			set panvar(dataLabel) $myData(0,$ix,iDataName)
			write_crvfit_data $pid $pNum 0 $xcv(fn,dataTmp)
			set panvar(gfit,dataIx,$pNum) $ix
			set panvar(gfit,$pNum,rmseLow) "99999"
			set panvar(gfit,callNum) 0
			gridfit_one_data 0 $pNum $gridParmList
			update_gridfit_panel $f $panvar(dataLabel) $pNum $gridParmList
			set panvar(gfit,$pNum,outrmse) ""
			set panvar(gfit,$pNum,rmseLow) ""
			set panvar(dataLabel) ""
			incr pNum 
		}
	}
	set panvar(gfit,dNumTot) $pNum

	# summarize and save results to a file
	gridfit_summary $f $pNum
	save_text $f.results.text $xcv(fn,resGridTmp)
}

#####################################################################
#
# User clicks the hold constant radio button 
#
proc bind_hold_constant { f cur } {
	global panvar
	
	if { $panvar(gfit,choice,$cur) == "yes" } {
		$f.p$cur.ub.entry configure -state normal
		$f.p$cur.inc.entry configure -state normal
	} else {
		$f.p$cur.ub.entry configure -state disabled 
		$f.p$cur.inc.entry configure -state disabled 
	}
}

#####################################################################
#
# Get the function parameters, then call fastcrvfit
#
proc gridfit_one_data { cur dNum gridParmList } {
	global panvar xcv 

	# We have processed all the parameters, use fastcrvfit to evaluate. 
	if {$cur == $panvar(parms,num)} {
		grid_evaluate $dNum $gridParmList
		incr panvar(gfit,callNum)
		return
	}

	# If current parameter is not held constant, then no changes required
	if {[string compare $panvar(gfit,choice,$cur) no] == 0} {
		set val [string trim $panvar(newParms,$cur)]
		if {[string length $val] == 0} {
			u_error "No value given for parameter $cur"
		}
		gridfit_one_data [expr $cur + 1] $dNum $gridParmList
		return
	}

	# User wants a constant parameter only
	set ub [string trim $panvar(gfit,ub,$cur)]
	set inc [string trim $panvar(gfit,inc,$cur)]
	if {[string length $ub] == 0 || [string length $inc] == 0} {
		set val "=$panvar(newParms,$cur)"
		set gridParmList [lreplace $gridParmList $cur $cur $val]
		gridfit_one_data [expr $cur + 1] $dNum $gridParmList
		return
	}

	# User wants a constant parameter over a specified range
	set val [string trim $panvar(newParms,$cur)]
	while { $val <= $ub } {
		lappend panvar(parmGrid) $val
		set gridParmList [lreplace $gridParmList $cur $cur "=$val"]
		gridfit_one_data [expr $cur + 1] $dNum $gridParmList
		set val [expr $val + $inc]
	}
}

#####################################################################
#
# we have a set of parameters in order to call fastcrvfit 
# keep track of the results
#
proc grid_evaluate { dNum gridParmList } {
	global panvar xcv myFunc myData

	# we have a complete parameter list, call fastcrvfit
	if { [call_fastcrvfit $gridParmList] == "FALSE" } {
		u_log "Unable to converge for DataSet: $myData(0,$dNum,iDataName)"
		return
	}

	# read best parameters from fastcrvfit and store them
	set f [open $xcv(fn,resShortTmp)]
	set c $panvar(gfit,callNum)
	set outLine "Output: "
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		gets $f line
		set nd [scan $line "%f" panvar(gfit,$dNum,$c,outparms,$np)]
		set outLine "$outLine $panvar(gfit,$dNum,$c,outparms,$np)"
	}

	gets $f line
	set nd [scan $line "%f" panvar(gfit,$dNum,$c,outrmse)]
	close $f
	set outLine "$outLine rmse=$panvar(gfit,$dNum,$c,outrmse)"
	u_log $outLine

	set panvar(gfit,$dNum,$c,parmList) $gridParmList
	set panvar(gfit,$dNum,outrmse) $panvar(gfit,$dNum,$c,outrmse)
	if { $panvar(gfit,$dNum,outrmse) < $panvar(gfit,$dNum,rmseLow) } {
		set panvar(gfit,$dNum,rmseLow) $panvar(gfit,$dNum,outrmse)
		set panvar(gfit,$dNum,callLow) $c
	}
}


#####################################################################
#
# Gridfit results panel 
#
proc show_gridfit_results_panel { f } {
	global xcv panvar myFunc

	if [winfo exists $f] {
		raise $f
		return
	}

	set tf $f.results
	toplevel $f

	frame $f.live
	pack $f.live -pady 5
	label $f.live.label -text "Calculating"
	label $f.live.dataset
	label $f.live.rmseVal -width 7 
	pack $f.live.label $f.live.dataset $f.live.rmseVal -side left
	label $f.live.rmseLabel -text "Lowest RMSE"
	label $f.live.rmseLow -width 7
	pack $f.live.label $f.live.rmseLow -side left

	frame $tf 
	pack $tf -pady 10 -padx 20 -expand true -fill both
	text $tf.text -relief raised -width 90 \
		-height 24 -bd 2 -yscrollcommand "$tf.scroll set"
	scrollbar $tf.scroll -command "$tf.text yview"
	pack $tf.scroll -side right -fill y
	pack $tf.text -side left -expand true -fill both

	frame $f.but
	pack $f.but -pady 10 -padx 20
	set outFile $xcv(fn,resGridTmp)
	button $f.but.save -text "Save to File" \
		-command "bind_save_file $outFile"
	button $f.but.plot -text "Plot Sum of Square Errors" \
		-command "grid_plot_rmse"
	button $f.but.show -text "Show Fit for each Dataset" \
		-command "grid_show_fits"
	button $f.but.quit -text "Dismiss" -command "destroy $f"
	pack $f.but.save $f.but.plot $f.but.show $f.but.quit -side left

	$tf.text insert 1.0 "Grid Fitting Results\n"
	$tf.text insert 2.0 "Label      SSE  Parameters\n"
	update
}

#####################################################################
#
# Show progress window to display calculation results
#
proc update_gridfit_panel { f dataset dNum gridParmList } {
	global panvar myFunc

	set tf $f.results

	$f.live.dataset configure -textvariable panvar(dataLabel)
#	$f.live.rmseVal configure -textvariable panvar(gfit,$dNum,outrmse)
#	$f.live.rmseLow configure -textvariable panvar(gfit,$dNum,rmseLow)

	# create summary line for this dataset 
	set hd [format "%5s %8.4g" \
		$panvar(dataLabel) $panvar(gfit,$dNum,rmseLow)]
	set outLine "$hd  "

	# add parameter values
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
	    set lab [lindex $myFunc($panvar(functionKey),parms) $np]
		set cLow $panvar(gfit,$dNum,callLow)
		set val $panvar(gfit,$dNum,$cLow,outparms,$np)
		set outLine "$outLine $lab:$val"
	}
	# show output line
	$tf.text insert 3.0 "$outLine\n"
	update
}

#####################################################################
#
# Display lowest rmse for gridfit 
#
proc gridfit_summary { f dNumTot } {
	global myFunc panvar vPlot xcv

	if { $panvar(gfit,callNum) <= 0 } {
		return
	}

	# Total all rmse for all the datasets 
	for {set c 0 } {$c < $panvar(gfit,callNum)} {incr c} {
		set panvar(gfit,$c,rmseTot) $panvar(gfit,0,$c,outrmse) 
		for {set n 1 } {$n < $dNumTot} {incr n} {
			set panvar(gfit,$c,rmseTot) [expr $panvar(gfit,$c,rmseTot) + $panvar(gfit,$n,$c,outrmse) ]
		}
	}

	# Write the results to a file, find the lowest rmse total 
	set outfile [open $xcv(fn,out_rmse) w]
	set rmseLow 999999
	for {set c 0 } {$c < $panvar(gfit,callNum)} {incr c} {
		set x [lindex $panvar(parmGrid) $c]
		puts $outfile "$x $panvar(gfit,$c,rmseTot)"
		if {$panvar(gfit,$c,rmseTot) < $rmseLow} {
			set rmseLow $panvar(gfit,$c,rmseTot)
			set cLow $c
			set panvar(cLow) $c
		}
	}

	# Find the parameters producing the lowest rmse total. 
	set parmStr ""
	set panvar(parms,best) $panvar(gfit,0,$cLow,parmList)
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		set panvar(parms,$np) [lindex $panvar(gfit,0,$cLow,parmList) $np]
		set p [lindex $panvar(gfit,0,$cLow,parmList) $np]
		if {[string index $p 0] == "="} {
			set aVal [string range $p 1 end]
			set aLab [lindex $myFunc($panvar(functionKey),parms) $np]
			set parmStr "$parmStr $aLab:$aVal "
		}
	}

	set tf $f.results
	set outLine "$rmseLow at $parmStr "
	$tf.text insert 2.0 "$outLine \n\n"
	$tf.text insert 2.0 "\nLowest sse (sum of the square of the error) for all selected data:\n"
	close $outfile
	$f.live.label configure -text "Done"
}

#####################################################################
#
# Get the function parameters, then call fastcrvfit
#
proc grid_plot_rmse { } {
	global panvar vPlot myData xcv

	set pid 2 
	set iFile $pid

	read_crvfit_data $xcv(fn,out_rmse) $iFile
	if { $myData($iFile,0,iDataNum) <= 1 } {
		u_error "Cannot plot with less than two values"
	}

	plot_newdata $pid 0 $iFile
	update
	do_plot $pid $iFile
	raise ".$xcv(win,plot,$pid)"
}

#####################################################################
#
# show how each data fits with the best parms from gridfit 
#
proc grid_show_fits { } {
	global panvar vPlot myData xcv

	set iFile 0 
	set pid 3
	set pNum 0
	set vPlot($pid,pNumTot) 0
	set vPlot($pid,pNumCur) 0
	create_plot_win $pid

	# set parameters
	set gridParmList $panvar(parms,best)

	# process each dataset
	for {set mix 0 } {$mix < $myData(0,iSetNum)} {incr mix} {
		if { $myData(0,$mix,gridFlag) == "TRUE" } {

			set vPlot($pid,$pNum,okFlag) "TRUE"
			plot_init_new $pid $pid $pNum $iFile 
			plot_set_data $pid $pid $pNum 0 0 $mix
			incr vPlot($pid,pNumTot)
			set vPlot($pid,$pNum,dNumCur) 0
			set vPlot($pid,$pNum,dNumTot) 1

			write_crvfit_data $pid $pNum 0 $xcv(fn,dataTmp)
			if { [call_fastcrvfit $gridParmList] == "FALSE" } {
				u_log "Unable to converge for DataSet: $myData(0,$dNum,iDataName)"
				return
			}

			set f [open $xcv(fn,resShortTmp)]
			for {set np 0} {$np < $panvar(parms,num)} {incr np} {
				if {[gets $f line] >= 0} {
					set nd [scan $line "%f %f" num std]
					set panvar(newParms,$np) $num
					set panvar(parms,$np) $num
				}
			}

			calc_fit_one $pid $pNum 0
			incr pNum 
		}
	}

	find_grid $pid 0
	plot_set_grid $pid $vPlot($pid,grid,row) $vPlot($pid,grid,col)
	calibrate_xy $pid
	do_plot $pid $iFile

	raise ".$xcv(win,plot,$pid)"

	# Put original parameters back
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		if [info exists panvar(parmsCopy,$np)] {
			set panvar(parms,$np) $panvar(parmsCopy,$np) 
			set panvar(newParms,$np) $panvar(parmsCopy,$np) 
		}
	}
}
