
#####################################################################
#
# xcrvfit main window widgets
#
#####################################################################


#####################################################################
#
# screen layout
#
proc panel_layout {f} {
	global xwin myFunc panvar

	set xwin(ctl) $f.ctl
	set xwin(data) $f.dat
	set xwin(fit) $f.fit
	set xwin(parms) $f.parms
	set xwin(buttons) $f.buts

	frame $xwin(ctl)
	pack $xwin(ctl) -side left -anchor nw
	u_log "Building xcrvfit main panel..."

	panel_menubar $f.menus
	panel_data $f.dat
	panel_fit_func $f.fit
	panel_fit_parms $f.parms 
	panel_buttons $f.buts 
}

#####################################################################
#
# create menubar
#
proc panel_menubar { f } {
    frame $f
	pack $f
	set mb $f.mbar
	frame $mb -relief raised -bd 2
	frame $f.dummy
	pack $mb $f -side top -fill x
	create_menus $mb
}

#####################################################################
#
# create menubar
#
proc create_menus {f} {
	global xcv panvar

#
# menubar entries
#
	menubutton $f.file -padx 10 -text File -underline 0 -menu $f.file.menu
	menubutton $f.settings -padx 10 -text "Settings" \
		-underline 0 -menu $f.settings.menu
	menubutton $f.options -padx 10 -text "Options" \
		-underline 0 -menu $f.options.menu
	menubutton $f.helpb -padx 10 -text "Help" -underline 0 -menu $f.helpb.menu
	pack $f.file $f.settings $f.options $f.helpb -side left
	tk_menuBar $f $f.file $f.settings $f.options $f.helpb
	focus $f
	
#
# File menu item
#
	set g $f.file.menu
	menu $g
	$g add command -label "About authors..." -command "pop_authors $f.authors"
	$g add separator
	$g add command -label "Exit" -command "bind_exit"
	
#
# Settings menu item
#
	set g $f.settings.menu
	menu $g
	$g add command -label "Plot settings for $xcv(win,plot,0)..." \
		-command "plot_settings 0 .$xcv(win,plot,0)-settings"
    $g add separator
	foreach i {1 2 3} {
		$g add command -label "Plot settings for $xcv(win,plot,$i)..." \
			-command "plot_settings $i .$xcv(win,plot,$i)-settings"
	}
    $g add separator
	foreach i {4 5} {
		$g add command -label "Plot settings for $xcv(win,plot,$i)..." \
			-command "plot_settings $i .$xcv(win,plot,$i)-settings"
	}
	
#
# Options menu item
#
	do_options_menu $f.options
	
#
# Help menu item
#
	set g $f.helpb.menu
	menu $g
	$g add command -label "Fitting Function Notes..." -command "bind_func_desc"
	$g add command -label "Website Help..." -command "pop_help $f.helpb.help"
}

#####################################################################
#
# data file layout 
#
proc panel_data { f } {
	global xcv panvar myData

	set panvar(data,format) crvfit

	frame $f
	pack $f -pady 15 -padx 20

	u_file_entry $f.file    "Data File:" 10 panvar(data,filename) 40 15 bind_dataFile 1
	pack $f.file -pady 3 

	frame $f.set
	pack $f.set
	label $f.set.lab -text "Data Set:"
	label $f.set.start -textvariable panvar(dataSet) -width 3
	label $f.set.prep -text "of"
	label $f.set.end -textvariable myData(0,iSetNum) -width 3
	pack $f.set.lab $f.set.start $f.set.prep $f.set.end -side left
	command_entry $f.set.dataSet "Label:" 9 "bind_dataSet" \
		-textvar panvar(dataLabel) -width 12
	pack $f.set.dataSet -side left

	frame $f.set2
	pack $f.set2 -pady 3
	button $f.set2.next -text "Next" -command "bind_next_dataSet 1"
	button $f.set2.prev -text "Previous" -command "bind_next_dataSet -1"
	pack $f.set2.next $f.set2.prev -side left
}

#####################################################################
#
# create fitting function area 
#
proc panel_fit_func { f } {
	global panvar myFunc xcv

	frame $f
	pack $f -pady 10

	set m $f.select
	frame $m
	pack $m 
	label $m.label -text "Fit Function:" -width 12
	menubutton $m.func -padx 9 -menu $m.func.menu -underline 0
	pack $m.label $m.func -side left

	menu $m.func.menu
	set count 0
	foreach group $xcv(function,list) {

		$m.func.menu add cascade -label $group -menu $m.func.menu.$group
		menu $m.func.menu.$group

		foreach type $xcv(function,$group) {
			$m.func.menu.$group add command -label $myFunc($type,name) \
				-command "bind_fit_func $type"
			incr count
		}
	}

	frame $f.note
	pack $f.note
	label $f.note.label -textvariable panvar(fitNote)
	pack $f.note.label 
}

#####################################################################
#
# set parameters for the fitting function selected 
#
proc panel_fit_parms {f} {
	global panvar xcv

	frame $f
	pack $f
	label $f.title -text "   Parameter        Hold Constant"
	pack $f.title -ipadx 0

	for {set np 0} {$np < $xcv(MAX_PARM_NUM)} {incr np} {

		command_entry $f.p$np a$np 5 "bind_parm $np" \
			-textvar panvar(newParms,$np) -width 10 
		pack $f.p$np

		set cf [frame $f.p$np.choices -borderwidth 2]
		pack $cf -side left -ipadx 5
		radiobutton $cf.0 -variable panvar(choice,$np) -text "yes" \
			-value "yes"
		pack $cf.0 -side left 
		radiobutton $cf.1 -variable panvar(choice,$np) -text "no" \
			-value "no"
		pack $cf.1 -side left 
		set panvar(choice,$np) "no"
	}
}

#####################################################################
#
# update results from calculations on the main panel 
#
proc panel_calc_update { } {
	global xwin panvar

	set f $xwin(buttons)

	if [ info exists panvar(rmse) ] {
		$f.rmse.label configure -text "Rmse......: " 
		$f.rmse.value configure -text $panvar(rmse)
	} else {
		$f.rmse.label configure -text " "
		$f.rmse.value configure -text " "
	}

	if [ info exists panvar(itNum) ] {
		$f.itNum.label configure -text "Iterations: " 
		$f.itNum.value configure -text $panvar(itNum)
	} else {
		$f.itNum.label configure -text " "
		$f.itNum.value configure -text " "
	}
}

#####################################################################
#
# create buttons area
#
proc panel_buttons {f} {
	global panvar xcv 

	frame $f -relief flat -bd 5
	pack $f -ipady 2 -expand 1 -anchor nw -fill x

	label $f.nop1
	pack $f.nop1

	frame $f.rmse
	pack $f.rmse
	label $f.rmse.label -text  " "
	label $f.rmse.value -width 12 -text " " -anchor w
	pack $f.rmse.label $f.rmse.value -side left

	frame $f.itNum
	pack $f.itNum
	label $f.itNum.label -text " "
	label $f.itNum.value -width 12 -text " " -anchor w
	pack $f.itNum.label $f.itNum.value -side left

	label $f.nop2
	pack $f.nop2

	frame $f.b1
	pack $f.b1
	button $f.b1.calc -text "Calc-rmse" -command {bind_calc_rmse}
	button $f.b1.it -text "Best Fit" -command {bind_iterate}
	button $f.b1.results -text "Results" \
		-command "bind_display_results 50 30"
	button $f.b1.quitB -text "Quit" -command "bind_exit" 
	pack $f.b1.calc $f.b1.it $f.b1.results $f.b1.quitB -side left
}

#####################################################################
#####################################################################
#####################################################################
#####################################################################


#####################################################################
#
# event procedures
#
#####################################################################

#####################################################################
#
# process the dataFile field
#
proc bind_dataFile { } {
	if { [read_data] == "TRUE" } {
		plot_newdata 0 0 0
	}
}

#####################################################################
#
# event procedure for changing the dataset field 
# 
proc bind_dataSet { } {
    global myData xcv panvar vPlot

	for {set mix 0} {$mix < $myData(0,iSetNum)} {incr mix} {
		if {$myData(0,$mix,iDataName) == $panvar(dataLabel)} {
			plot_dataset 0 0 $mix
			u_log "Dataset is changed to '$panvar(dataLabel)'."
			break
		}
	}
}

#####################################################################
#
# user hits the Next Data Set button
#
proc bind_next_dataSet { inc } {
	global xcv xwin panvar myData vPlot

	# check data
	set iFile 0
	if ![info exists myData($iFile,iSetNum)] {
		u_error "Data File not specified."
	}
	if {$myData($iFile,iSetNum) <= 0} {
		u_error "No current datasets."
	}

	set mix [get_next_dataSet $inc $iFile]
	plot_dataset 0 $iFile $mix

	u_log "Next dataset number is changed to '$panvar(dataLabel)'."
}

#####################################################################
#
# calculate next dataset index
#
proc get_next_dataSet { inc iFile } {
	global panvar myData

	set mix [expr $panvar(dataIndex) + $inc]
	if {$mix >= $myData($iFile,iSetNum)} {
		set mix 0
	}
	if {$mix < 0} {
		set mix [expr $myData($iFile,iSetNum) - 1]
	}
	return $mix
}

#####################################################################
#
# update current plot 
#
proc update_plot_event { } {
	global xcv xwin panvar

	set pid 0
	plot_set_data $pid $pid $pNum $dNum $panvar(dataIndex)
	set panvar(dataSet) [expr $panvar(dataIndex) + 1]

	do_plot $pid 0
}


#####################################################################
#
# event procedure for changing the fitting function selection 
#
proc bind_fit_func { key } {
	global panvar xwin xcv myFunc

	set panvar(functionKey) $key
	set panvar(fitNote) $myFunc($key,note)
	set panvar(parms,num) [llength $myFunc($key,parms)] 

	u_log "Fitting function set to '$key'."

	$xwin(fit).select.func configure -text $myFunc($key,name)

	for {set i 0} {$i < $xcv(MAX_PARM_NUM)} {incr i} {
		pack forget $xwin(parms).p$i
	}

	set i 0
	set labels $myFunc($key,parms)
	foreach item $labels {
		set panvar(parmLabel,$i) $item
		$xwin(parms).p$i.label configure -text $item
		$xwin(parms).p$i.entry config -textvariable panvar(newParms,$i) 
		pack $xwin(parms).p$i
		incr i
	}
	if [info exists panvar(rmse)] { unset panvar(rmse) }
	if [info exists panvar(itNum)] { unset panvar(itNum) }
	focus $xwin(parms).p0.entry
}

#####################################################################
#
# event procedures for each parameter item up to max num of parameters
#
proc bind_parm { item } {
	global xwin panvar vPlot

	# set cursor focus on next parameter
	incr item
	if {$item == $panvar(parms,num)} {
		set item 0
	}
	focus $xwin(parms).p$item.entry
}

#
# check fitting function parameters
#
proc check_parm_state { } {
    global panvar

    for {set ix 0} {$ix < $panvar(parms,num)} {incr ix} {
		if [catch {set a [expr double($panvar(newParms,$ix))]} err] {
			return $ix
		}
	}

    for {set ix 0} {$ix < $panvar(parms,num)} {incr ix} {
		set panvar(parms,$ix) [expr double($panvar(newParms,$ix))]
	}
	return $ix 
}

#####################################################################
#
# user hits the calc-rmse button
#
proc bind_calc_rmse { } {
	global xcv panvar

	# check data
	if {$panvar(dataIndex) < 0} {
		u_error "Unable to run numerical calculations, input data not read."
	}

	set rc [check_parm_state]
	if { $rc < $panvar(parms,num) } {
		u_error "Illegal parameter value at '$panvar(parmLabel,$rc)'"
	}

	set pid 0
	calc_rmse
	panel_calc_update
	do_plot $pid 0

	# update output window if it is open
	if [winfo exists ".$xcv(win,results)"] {
		bind_display_results 50 30
	}

	# Make sure output plot and main window are viewable
	raise ".$xcv(win,plot,$pid)" 
	raise ".$xcv(win,main)"
}

#####################################################################
#
# user hits the Best Fit button
#
proc bind_iterate { } {
	global panvar xcv

	# check data
	if {$panvar(dataIndex) < 0} {
		u_error "Unable to run numerical calculations, input data not read."
	}

	set rc [check_parm_state]
	if { $rc < $panvar(parms,num) } {
		u_error "Illegal parameter value at '$panvar(parmLabel,$rc)'"
	}

	set pid 0
	calc_iterate
	panel_calc_update
	do_plot $pid 0

	# update output window if it is open
	if [winfo exists ".$xcv(win,results)"] {
		bind_display_results 50 30
	}

	# Make sure output plot and main window are viewable
	raise ".$xcv(win,plot,$pid)" 
	raise ".$xcv(win,main)"
}

#####################################################################
#
# user clicks the Display Results button 
#
proc bind_display_results { wd ht } {
	global xcv panvar

	set file1 $panvar(fn,resSave)
    if ![file exists $file1] {
		u_error "Output File '$file1' does not exist"
	}

	set f1 ".$xcv(win,results)"
	set tf $f1.msg
	if ![winfo exists $f1] {
		toplevel $f1

		frame $f1.msg
		pack $f1.msg
	
   		text $tf.text -relief raised -bd 2 -state normal -height $ht \
			-yscrollcommand "$tf.scroll set"
		scrollbar $tf.scroll -command "$tf.text yview"
		pack $tf.scroll -side right -fill y
		pack $tf.text -side left

		frame $f1.but
		pack $f1.but
		button $f1.but.save -text "Save" \
			-command "bind_save_file $file1"
		button $f1.but.quit -text "Dismiss" -command "destroy $f1"
		pack $f1.but.save $f1.but.quit -side left -pady 5
	}

	$tf.text delete 1.0 end
	set f [open $file1]
	while {![eof $f]} {
		$tf.text insert end [read $f 1000]
	}
	close $f
}

######################################################################
#
# ask user for results file 
#
proc bind_save_file {inFile} {
	global panvar xcv

	set f1 ".$xcv(win,savetxt)"
	if [winfo exists $f1] {
		destroy $f1
	}
	toplevel $f1

	set panvar(outputFile) "" 
	command_entry $f1.save "Save to File: " 15 {} \
		-textvar panvar(outputFile) -width 20
	pack $f1.save -pady 5 -padx 20
	frame $f1.but
	pack $f1.but
	button $f1.but.save -text "OK" \
		-command "save_file $inFile; destroy $f1"
	button $f1.but.quit -text "Dismiss" -command "destroy $f1"
	pack $f1.but.save $f1.but.quit -side left -pady 5
}

######################################################################
#
# save results file 
#
proc save_file { inFile } {
	global panvar

	set outFile $panvar(outputFile)
    if { [file exists $outFile] } {
		error "Output file '$outFile' already exists, enter a new output file."
	}
	exec touch $outFile
    if { ![file exists $outFile] } {
		error "Cannot create output file '$outFile'."
	}
	exec cat $inFile > $outFile
}

#####################################################################
#
# user quits xcrvfit 
# remove any temporary files created
#
proc bind_exit {} {
	global xcv 
	exec rm -f $xcv(fn,resShortTmp) $xcv(fn,resLongTmp)
	exec rm -f $xcv(fn,resGridTmp) $xcv(fn,dataTmp)
	exit
}


#####################################################################
#####################################################################
#####################################################################
#####################################################################

#####################################################################
#
# change the cursor to notify the user of program state 
#
proc show_cursor { c_text c_window } {
	global xwin panvar 

	$xwin(ctl) config -cursor $c_window

#	$xwin(data).file.b.entry config -cursor $c_text
#	$xwin(data).set.dataSet.entry config -cursor $c_text
#	$xwin(data).set.offset.entry config -cursor $c_text
#	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
#		$xwin(parms).p$np.entry config -cursor $c_text
#	}
	update
}

#
# About the program window
#
proc pop_authors { f } {
	global xcv

	toplevel $f 

	label $f.vers -text "xcrvfit"
	label $f.date -text "$xcv(VERSION) $xcv(VERSION_DATE)"
	label $f.nop1
	label $f.aut \
		-text "Author: Robert Boyko / Brian Sykes"
	label $f.purpose \
		-text "Purpose: General curve fitting program"
	label $f.nop2
	label $f.msg1 \
		-text " Note: This project is funded by\
		the Protein Engineering Network Centres of"
	label $f.msg2 \
		-text " Excellence and the Canadian Institutes\
		of Health Research (CIHR group)"
	label $f.nop3
	label $f.msg3 -text " Email - robert.boyko@ualberta.ca"
	button $f.but -text "Ok" -command "destroy $f"
	label $f.nop4
	pack $f.vers $f.date $f.nop1 $f.aut \
		$f.purpose $f.nop2
	pack $f.msg1 $f.msg2 -padx 10
	pack $f.nop3 $f.msg3 $f.nop4 -padx 10
	pack $f.but -side top
}

#
# Help window
#
proc pop_help { f } {
	global xcv

	toplevel $f 

	set message "All help is accessible via your favorite web browser at:

$xcv(dir,help) (locally)
or the URL
$xcv(url,help)
		
Here you will find information on data file formats, details on
fitting function parameters, troubleshooting, overlaying plots,
adding your own functions, etc."


	label $f.msg -text $message
	pack $f.msg -padx 10 -pady 10
	button $f.but -text "Ok" -command "destroy $f"
	pack $f.but -side top
}


#
# save curve fit plot to a file
#
proc pop_save_plot { pid } {
	global xcv xwin panvar

	set f ".$xcv(win,saveplot)"
	if [winfo exists $f] {
		destroy $f
	}
	toplevel $f

	set formats {png tiff gif jpeg postscript}
	set panvar(plot,png) png
	set panvar(plot,tiff) tiff
	set panvar(plot,gif) gif
	set panvar(plot,jpeg) jpeg 
	set panvar(plot,postscript) ps
	set panvar(plot,suffix) $panvar(plot,png)

	set fit $panvar(functionKey)
	set panvar(plot,fn) "out.$fit.$panvar(dataLabel).$panvar(plot,suffix)"

	frame $f.msg
	pack $f.msg -pady 10 -padx 10
	label $f.msg1 -text "Select format for xcrvfit plot. You must have the tcl"
	label $f.msg2 -text "Img package to save in formats other than postscript."
	pack $f.msg1 $f.msg2

	frame $f.check
	pack $f.check -pady 10
	foreach item $formats {
		radiobutton $f.check.$item -variable panvar(plot,suffix) -text $item \
			-value $panvar(plot,$item) -selectcolor red -command plot_set_fn
		pack $f.check.$item -anchor w
	}

	command_entry $f.fn "Save to file: " 15 {} -textvar panvar(plot,fn) -width 30
	pack $f.fn -padx 10 -pady 10

	frame $f.but
	pack $f.but -pady 10
	button $f.but.ok -text "ok" -command "plot_save $pid; destroy $f"
	button $f.but.quit -text "Dismiss" -command "destroy $f"
	pack $f.but.ok $f.but.quit -side left 
}

# 
# Set output file name according to format selected.
#
proc plot_set_fn {} {
	global panvar 
	set new [file rootname $panvar(plot,fn)]
	set panvar(plot,fn) "$new.$panvar(plot,suffix)"
}
