
#
# xcrvfit Plotting routines 
#

###############################################################################
#
# We create a plot window with one plot using first dataset  
#
proc plot_newdata { pid dNum iFile } {
	global vPlot

	set mix 0
	set pNum 0

	set vPlot($pid,pNumTot) 0
	set vPlot($pid,$pNum,dNumTot) 0
	set vPlot($pid,pNumCur) 0
	set vPlot($pid,$pNum,dNumCur) 0
	set vPlot($pid,grid,row) 1
	set vPlot($pid,grid,col) 1

	create_plot_win $pid
	plot_init_new $pid $pid $pNum $iFile
	
	# add the first data set
	plot_set_data $pid $pid $pNum $dNum $iFile $mix
	set vPlot($pid,$pNum,okFlag) "TRUE"
	set vPlot($pid,pNumTot) 1
	set vPlot($pid,$pNum,dNumTot) 1
	plot_redraw $pid $pNum $mix $iFile
}

###############################################################################
#
# We create a plot window with one plot using first dataset  
#
proc plot_dataset { pid iFile mix } {
	global vPlot

	set pNum $vPlot($pid,pNumCur)
	set dNum $vPlot($pid,$pNum,dNumCur)
	plot_set_data $pid $pid $pNum $dNum $iFile $mix

	plot_panel_update $pid $pNum $mix
	calibrate_xy $pid
	do_plot $pid $iFile 
}

###############################################################################
#
# Add a new plot to the current plot window 
#
proc plot_add { pid mix } {
	global vPlot

	set iFile 0
	set dNum 0
	set pNum $vPlot($pid,pNumTot)
	incr vPlot($pid,pNumTot)
	set vPlot($pid,$pNum,dNumTot) 1
	set vPlot($pid,$pNum,dNumCur) 0

	set vPlot($pid,$pNum,okFlag) "TRUE"
	plot_init_new $pid $pid $pNum $iFile
	plot_set_data $pid $pid $pNum $dNum $iFile $mix
	plot_redraw $pid $pNum $mix $iFile
}

###############################################################################
#
# We create a plot window with one plot using first dataset  
#
proc plot_remove { pid mix } {
	global vPlot

	set iFile 0
	set pNum $vPlot($pid,pNumCur)
	set vPlot($pid,$pNum,okFlag) "FALSE"
	plot_redraw $pid $pNum $mix $iFile
}

###############################################################################
#
# Re-init plotting canvas 
#
proc plot_reset { pid } {
	global vPlot panvar xcv

	set msg "\nAre you sure you want to re-initialize your plotting window?\n"
	set resp [ pop_question $msg ]
	if { $resp == 1 } {return}

	set pNum 0
	set dNum 0
	set mix 0
	set iFile 0
	plot_init_new $pid $pid $pNum 0 
	plot_set_data $pid $pid $pNum $dNum $iFile $mix

	set vPlot($pid,$pNum,okFlag) "TRUE"
	set vPlot($pid,pNumTot) 1
	set vPlot($pid,$pNum,dNumTot) 1
	set vPlot($pid,pNumCur) 0
	set vPlot($pid,$pNum,dNumCur) 0
	set vPlot($pid,pNumCurText) $vPlot($pid,$pNum,0,data_label_text)
	plot_redraw $pid $pNum $mix $iFile
}

###############################################################################
#
# user changes the size of a plot window 
#
proc plot_resize_event { pid } {
	global vPlot xcv panvar

	if { $panvar(resizeFlag) == "FALSE" } {
		set panvar(resizeFlag) "TRUE"

		# get coordinates of plot window
		set pWin .$xcv(win,plot,$pid)
		set panvar($pid,canvas_geometry) [wm geometry $pWin]
		set vPlot($pid,canvas_geometry) $panvar($pid,canvas_geometry)
	
		calibrate_geometry $pid
		calibrate_xy $pid 
		if { $pid == 2 } {
			do_plot $pid 2
		} else {
			do_plot $pid 0 
		}
		update
		set panvar(resizeFlag) "FALSE"
	}
}

###############################################################################
#
# save graphics in selected format 
#
proc plot_save { pid } {
	global vPlot panvar xcv

	set fn "$xcv(MY_HOME)/$panvar(plot,fn)"
	if { $panvar(plot,suffix) == "ps" } {
		u_log "Saving plot to postscript file '$fn'"
		if [catch { set rc [$vPlot($pid,canvas) postscript -file $fn] } err] {
			u_error "Problems Saving postscript plot: '$err'"
		}
	} else {
		package require Img
		if [catch { set cv [image create photo -format window -data $vPlot($pid,canvas)] } err] {
			u_error "Cannot create photo from plot window. Do you have the tcl Img extension?"
		}
		u_log "Saving plot to file '$fn'"
		if [catch { set rc [$cv write -format $panvar(plot,suffix) $fn] } err] {
			u_error "Problems Saving plot: '$err'"
		}
	}
}

###############################################################################
#
# Generic update plotting window 
#
proc plot_redraw { pid pNum mix iFile } {
	global vPlot

	find_grid $pid 0
	plot_set_grid $pid $vPlot($pid,grid,row) $vPlot($pid,grid,col)
	plot_panel_update $pid $pNum $mix
	calibrate_xy $pid
	do_plot $pid $iFile 
}

#####################################################################
# 
# Determine the layout of our multiple plots 
#
proc find_grid { pid start } {
	global vPlot panvar

	set plotNum $start 
	set vPlot($pid,grid,row) 1
	set vPlot($pid,grid,col) 1
	set totPlot 1
	for {set j 0} {$j < $vPlot($pid,pNumTot)} {incr j} {
		if { $vPlot($pid,$j,okFlag) == "TRUE" } {
			incr plotNum
		}
		if { $plotNum > $totPlot } {
			if {$vPlot($pid,grid,row) >= $vPlot($pid,grid,col)} {
				incr vPlot($pid,grid,col)
			} else {
				incr vPlot($pid,grid,row)
			}
			set totPlot [expr $vPlot($pid,grid,row) * $vPlot($pid,grid,col)] 
		}
	}
}

###############################################################################
#
# Set up each plot window to have n rows and m columns of plots.
#
proc plot_set_grid { pid row col } {
    global vPlot xcv

	# set initial values for grid dimensions 
	set vPlot($pid,grid,row) $row
	set vPlot($pid,grid,col) $col

	# keep track of valid current plots in this plot window
	plot_set_valid $pid

	# set up each plot 
	set yInc [expr 100 / $row ]
	set xInc [expr 100 / $col ]
	set y0 0
	set y1 $yInc

	set count 0
	for {set i 0} {$i < $row} {incr i} {
		set x0 0
	    set x1 $xInc
		for {set j 0} {$j < $col} {incr j} {
			if {$count >= $vPlot(valid,total)} {
				return
			}
			set vNum $vPlot(valid,$count)
			init_one_plot $pid $vNum $x0 $x1 $y0 $y1
			set x0 $x1
			set x1 [expr $x1 + $xInc ]
			incr count 
		}
		set y0 $y1
		set y1 [expr $y1 + $yInc ]
    }
}

###############################################################################
#
# keep track of valid current plots in this plot window
#
proc plot_set_valid { pid } {
    global vPlot

	set count 0
	for {set j 0} {$j < $vPlot($pid,pNumTot)} {incr j} {
		if { $vPlot($pid,$j,okFlag) == "TRUE" } {
			set vPlot(valid,$count) $j
			incr count
		}
	}
	set vPlot(valid,total) $count
}

###############################################################################
#
# calculate plot parameters for one plot based on given dimensions.
# the dimensions are in percent values of the total plot window.
#
proc init_one_plot { pid pNum left right top bottom } {
	global vPlot panvar

	set vPlot($pid,$pNum,percent_left) $left
	set vPlot($pid,$pNum,percent_right) $right
	set vPlot($pid,$pNum,percent_top) $top
	set vPlot($pid,$pNum,percent_bottom) $bottom

	# set default plot attributes
	set attribList { x_tics y_tics \
		x_label_text x_label_x_offset x_label_y_offset x_label_color \
		y_label_text y_label_x_offset y_label_y_offset y_label_color \
		graph_label_text graph_label_x_offset graph_label_y_offset graph_label_color \
		x_format y_format x_border y_border axis_color canvas_font \
		margin_left margin_right margin_top margin_bottom }

	foreach item $attribList {
		if ![info exists vPlot($pid,$pNum,$item)] {
			set vPlot($pid,$pNum,$item) $panvar($item)
		}
	}
}

###############################################################################
#
# Update xcrvfit panel when we add/change the input data. 
#
proc plot_panel_update { pid pNum mix } {
	global xcv vPlot myData panvar 

	# set values on main window
	set panvar(dataIndex) $mix
	set panvar(dataSet) [expr $mix + 1]
	set panvar(dataLabel) $myData(0,$mix,iDataName)
	set panvar(data_label_text) $myData(0,$mix,iDataName)
	set panvar($pid,gridSize) "$vPlot($pid,grid,row) X $vPlot($pid,grid,col)"

	foreach item {x_min x_max y_min y_max} {
		set panvar($item) $myData(0,$item)
	}

	# if multiple datasets, enable next and previous buttons
	set f ".$xcv(win,main).frame.dat.set2"
	if { $myData(0,iSetNum) > 1 } {
		$f.next configure -state normal
		$f.prev configure -state normal
	} else {
		$f.next configure -state disabled
		$f.prev configure -state disabled
	}
}

###############################################################################
#
# We create a new plot in the current plot window 
#
proc plot_init_new { pid iStart pNum iFile } {
	global vPlot myData panvar xcv

	# Set default limits based on current data
	foreach item {x_min x_max y_min y_max bounded_left bounded_right} {
		set vPlot($pid,$pNum,$item) $myData($iFile,$item)
	}
	set vPlot($pid,$pNum,bounded_flag) 0

	# set default plot attributes
	set attribList { x_tics y_tics \
		x_label_text x_label_x_offset x_label_y_offset x_label_color \
		y_label_text y_label_x_offset y_label_y_offset y_label_color \
		graph_label_text graph_label_x_offset graph_label_y_offset graph_label_color \
		x_format y_format x_border y_border axis_color canvas_font \
		bounded_color margin_left margin_right margin_top margin_bottom }
	foreach item $attribList {
		if ![info exists vPlot($pid,$pNum,$item)] {
			set vPlot($pid,$pNum,$item) $vPlot($iStart,$item)
		}
	}
}

###############################################################################
#
# Associate data with a plot entity
#
proc plot_set_data { pid iStart pNum dNum iFile mix } {
	global vPlot myData panvar xcv

	# Store the index and name
	set vPlot($pid,$pNum,$dNum,mix) $mix 
	set vPlot($pid,$pNum,$dNum,data_label_text) $myData($iFile,$mix,iDataName) 

	# Here is our list of attributes
	set attribList {fit_color fit_points fit_line_width data_color \
		data_label_color data_label_x_offset data_label_y_offset \
		data_symbol}
	foreach item $attribList {
		if ![info exists vPlot($pid,$pNum,$dNum,$item)] {
			set vPlot($pid,$pNum,$dNum,$item) $vPlot($iStart,$item)
		}
	}
}

###############################################################################
#
# flash data is visual cue
#
proc blink_data { pid pNum dNum ms } {
    global vPlot xcv xwin panvar myData

	set old $vPlot($pid,$pNum,$dNum,data_color)
	set vPlot($pid,$pNum,$dNum,data_color) white
	draw_data $pid $pNum $dNum 0
	update idletasks
	after $ms
	set vPlot($pid,$pNum,$dNum,data_color) $old
	draw_data $pid $pNum $dNum 0
	after $ms
	update idletasks
}

###############################################################################
###############################################################################
###############################################################################

###############################################################################
#
# A change has been made in the dimensions along x or y.
# Re-calculate the appropriate plotting variables.
#
proc calibrate_xy { pid } {
	global vPlot

	for {set j 0} {$j < $vPlot($pid,pNumTot)} {incr j} {

		if [info exists vPlot($pid,$j,okFlag)] {
			if { $vPlot($pid,$j,okFlag) == "FALSE" } {
				continue
			}
		}

		# change percent values for margins to number of pixels
		set vPlot($pid,$j,pix_left) [expr $vPlot($pid,canvas_width) * 0.01 * \
			$vPlot($pid,$j,percent_left) + $vPlot($pid,$j,margin_left)]
		set vPlot($pid,$j,pix_right) [expr $vPlot($pid,canvas_width) * 0.01 * \
			$vPlot($pid,$j,percent_right) - $vPlot($pid,$j,margin_right)] 
		set vPlot($pid,$j,pix_top) [expr $vPlot($pid,canvas_height) * 0.01 * \
			$vPlot($pid,$j,percent_top) + $vPlot($pid,$j,margin_top)]
		set vPlot($pid,$j,pix_bottom) [expr $vPlot($pid,canvas_height) * 0.01 * \
			$vPlot($pid,$j,percent_bottom) - $vPlot($pid,$j,margin_bottom)]
	
		set vPlot($pid,$j,x_len) [expr $vPlot($pid,$j,pix_right) - $vPlot($pid,$j,pix_left)]
		set vPlot($pid,$j,y_len) [expr $vPlot($pid,$j,pix_bottom) - $vPlot($pid,$j,pix_top)]
		set vPlot($pid,$j,x_ref) $vPlot($pid,$j,pix_left)
		set vPlot($pid,$j,y_ref) $vPlot($pid,$j,pix_bottom)
	
		# update x/y ratio
		set vPlot($pid,$j,x_ratio) [expr $vPlot($pid,$j,x_len) / \
			double($vPlot($pid,$j,x_max) - $vPlot($pid,$j,x_min))]
		set vPlot($pid,$j,y_ratio) [expr $vPlot($pid,$j,y_len) / \
			double($vPlot($pid,$j,y_max) - $vPlot($pid,$j,y_min))]
		plot_set_axis $pid $j
	}
}

###############################################################################
#
# determine position of axis on a plot 
#
proc plot_set_axis { pid pNum } {
	global vPlot panvar

	# x-axis is at the top
	if { $vPlot($pid,$pNum,y_max) <= 0 } { 
		set vPlot($pid,$pNum,y_origin) $vPlot($pid,$pNum,pix_top)
		return
	}

	# x-axis is at the bottom
	if { $vPlot($pid,$pNum,y_min) >= 0 } { 
		set vPlot($pid,$pNum,y_origin) $vPlot($pid,$pNum,pix_bottom)
		return
	}

	# x-axis is in the middle if abs(y_min) = y_max
	set abs_y_min [expr 0 - $vPlot($pid,$pNum,y_min)]
	if {$vPlot($pid,$pNum,y_max) == $abs_y_min} {
		set vPlot($pid,$pNum,y_origin) \
			[expr 0.5 * ($vPlot($pid,$pNum,pix_bottom) + $vPlot($pid,$pNum,pix_top))]
	} else {
		set vPlot($pid,$pNum,y_origin) $vPlot($pid,$pNum,pix_bottom)
	}
}

###############################################################################
#
# Routine to re-calculate fitting function values (eg, change in x-axis) 
# for all plots in a plotting window
#
proc calibrate_fit { pid pNum } {
	global panvar vPlot myData

	set dNum $vPlot($pid,$pNum,dNumTot)
	for {set j 0} {$j < $dNum} {incr j} {
		for {set k 0} {$k < $panvar(parms,num)} {incr k} {
			if [info exists vPlot($pid,$pNum,$j,parms,$k)] {
				set panvar(parms,$k) $vPlot($pid,$pNum,$j,parms,$k)
			}
		}
		calc_fit_one $pid $pNum $j
	}
}

###############################################################################
#
# Set variables pertaining to plot window size 
#
proc calibrate_geometry { pid } {
	global vPlot panvar 

	set cVar [split $vPlot($pid,canvas_geometry) x+-]
	set vPlot($pid,canvas_width) [lindex $cVar 0]
	set vPlot($pid,canvas_height) [lindex $cVar 1]
	$vPlot($pid,canvas) configure -width $vPlot($pid,canvas_width) 
	$vPlot($pid,canvas) configure -height $vPlot($pid,canvas_height) 
}

###############################################################################
###############################################################################
###############################################################################

###############################################################################
#
# calc fitting function values for one data set of one plot
# 
proc calc_fit_one { pid pNum dNum } {
	global panvar vPlot myData

	set mix $vPlot($pid,$pNum,$dNum,mix)

	# save function and parameters we use to generate the fitting curve
	set vPlot($pid,$pNum,$dNum,parms,functionKey) $panvar(functionKey)
	for {set np 0} {$np < $panvar(parms,num)} {incr np} {
		 set vPlot($pid,$pNum,$dNum,parms,$np) $panvar(parms,$np)
	}

	if {$myData(0,$mix,nCol) == 2} {
		calc_fit_2col $pid $pNum $dNum
	} else {
		calc_fit_3col $pid $pNum $dNum
	}
}

###############################################################################
#
# calc fitting function on the screen for 2 column data 
# 
proc calc_fit_2col { pid pNum dNum } {
	global vPlot panvar

	set np $vPlot($pid,$pNum,$dNum,fit_points)
	set x $vPlot($pid,$pNum,x_min)
	set xInc [expr ($vPlot($pid,$pNum,x_max) - $vPlot($pid,$pNum,x_min)) / double($np)]

	incr np
	for {set j 0} {$j < $np} {incr j} {
		if [catch {set y [fnf $x 0.0]} err] {
			set msg " **Warning: '$err'.\n  Fit function undefined at x='$x'."
			u_log $msg
			puts stdout $msg
			set vPlot($pid,$pNum,$dNum,fitx,$j) 0
			set vPlot($pid,$pNum,$dNum,fity,$j) 0
			set vPlot($pid,$pNum,$dNum,valid,$j) 0
		} else {
			set vPlot($pid,$pNum,$dNum,fitx,$j) $x
			set vPlot($pid,$pNum,$dNum,fity,$j) $y
			set vPlot($pid,$pNum,$dNum,valid,$j) 1
		}
		set x [expr $x + $xInc]
	}
}

###############################################################################
#
# calc fitting function on the screen for 3 column data 
# 
proc calc_fit_3col { pid pNum dNum } {
	global panvar vPlot myData

	set np $vPlot($pid,$pNum,$dNum,fit_points)
	set xMin $vPlot($pid,$pNum,x_min)
	set xMax $vPlot($pid,$pNum,x_max)
	set xInc [expr ($xMax - $xMin) / double($np)]

	set mix $vPlot($pid,$pNum,$dNum,mix)
	set ub [expr $myData(0,$mix,iDataNum) - 1]
	set tSlope [expr ($myData(0,$mix,$ub,td) - $myData(0,$mix,0,td)) / double($myData(0,$mix,$ub,xd) - $myData(0,$mix,0,xd))]
	set b [expr $myData(0,$mix,0,td) - $tSlope * $myData(0,$mix,0,xd)]

	set xc $xMin
	incr np
	for {set j 0} {$j < $np} {incr j} {
		set x [expr $b * $xc / (1.0 - $tSlope * $xc)]
		set t [expr $tSlope * $x + $b]
		if [catch {set y [fnf $x $t]} err] {
			set msg " **Warning: '$err'.\n  Fit function not defined at x='$x'."
			u_log $msg
			puts stdout $msg
			set vPlot($pid,$pNum,$dNum,fitx,$j) 0
			set vPlot($pid,$pNum,$dNum,fity,$j) 0
			set vPlot($pid,$pNum,$dNum,valid,$j) 0
		} else {
			set xPlot [expr $x / $t]
			set vPlot($pid,$pNum,$dNum,fitx,$j) $xPlot
			set vPlot($pid,$pNum,$dNum,fity,$j) $y
			set vPlot($pid,$pNum,$dNum,valid,$j) 1
		}
		set xc [expr $xc + $xInc]
	}
}

###############################################################################
###############################################################################
###############################################################################


###############################################################################
# 
# Allow user to constrain the data 
#
proc plot_set_bounds {pid f} {
	global panvar vPlot xcv myData

	set padx 10
	set panvar(plotWinId) 0
	set panvar($pid,settings) DATA
	toplevel $f

	set mix $panvar(dataIndex)
	set first 0
	set last [expr $myData(0,$mix,iDataNum) - 1]
	set panvar(bounded_left) [bounded_convert 0 $mix $first]
	set panvar(bounded_right) [bounded_convert 0 $mix $last]

	label $f.wind -text "  Data to use in fitting calculations "
	pack $f.wind -side top -pady 20

	frame $f.bd
	pack $f.bd -anchor nw -padx $padx

	set f2 $f.bd.left
	frame $f2
	command_entry $f2.lb "Lower Bound for x: " 20 \
		"bounded_set $pid left " -textvar panvar(bounded_left) -width 9
	frame $f2.but
	button $f2.but.up -text "Increase" -command "bounded_new $pid left 1"
	button $f2.but.dn -text "Decrease" -command "bounded_new $pid left -1"
	pack $f2.but.up $f2.but.dn  -side left
	pack $f2.lb $f2.but  -padx 20 -side left
	pack $f2

	set f3 $f.bd.right
	frame $f3
	command_entry $f3.ub "Upper Bound for x: " 20 \
		"bounded_set $pid right" -textvar panvar(bounded_right) -width 9
	frame $f3.but
	button $f3.but.up -text "Increase" -command "bounded_new $pid right 1"
	button $f3.but.dn -text "Decrease" -command "bounded_new $pid right -1"
	pack $f3.but.up $f3.but.dn  -side left
	pack $f3.ub $f3.but  -padx 20 -side left
	pack $f3

	set_color $f.bd.color $pid "Bounded Color" 14 \
		panvar(bounded_color) "bounded_color"
	pack $f.bd.color -pady 10

	button $f.but -text "Dismiss" -command "destroy $f"
	pack $f.but -side top -pady 10
}

#####################################################################
# 
# Treat 2 column and 3 column data differently 
#
proc bounded_convert { iFile mix ix } {
	global myData

	if {$myData($iFile,$mix,nCol) == 2} {
		return $myData($iFile,$mix,$ix,xd)
	} else {
		return [expr $myData($iFile,$mix,$ix,xd) / $myData($iFile,$mix,$ix,td)]
	}
}

#####################################################################
# 
# Event procedure when user hits Increase or Decrease Buttons 
#
proc bounded_new { pid side inc } {
	global panvar xcv myData

	set mix $panvar(dataIndex)
	if { $inc > 0 } {
		for {set i 0} {$i < $myData(0,$mix,iDataNum)} {incr i} {
			set xVal [bounded_convert 0 $mix $i]
			if { $xVal > $panvar(bounded_$side) } {
				set panvar(bounded_$side) $xVal 
				bounded_set $pid $side
				return	
			}
		}
	}
	if { $inc < 0 } {
		set i [expr $myData(0,$mix,iDataNum) - 1]
		while {$i >= 0} {
			set xVal [bounded_convert 0 $mix $i]
			if { $xVal < $panvar(bounded_$side) } {
				set panvar(bounded_$side) $xVal 
				bounded_set $pid $side
				return	
			}
			set i [expr $i - 1]
		}
	}
}

#####################################################################
#
# limit field binding
#
proc bounded_set { pid side } {
	global vPlot xcv myData panvar

	set pNum $vPlot($pid,pNumCur)
	if { $panvar(bounded_left) >= $panvar(bounded_right) } {
		u_error "Boundary violation"
	}

	set vPlot($pid,$pNum,bounded_$side) $panvar(bounded_$side)
	set vPlot($pid,$pNum,bounded_flag) 1
	do_plot $pid 0
}

#####################################################################
#####################################################################
#####################################################################

###############################################################################
#
# get the default plotting variables 
#
proc get_plot_defaults { pid fc } {
	global vPlot xcv

	set_val vPlot($pid,canvas_geometry)	$fc canvas_geometry "600x500+600+30"
    set_val vPlot($pid,canvas_bg_color)	$fc canvas_bg_color beige
	set_val vPlot($pid,canvas_font)		$fc canvas_font		fixed

	set_val vPlot($pid,font_name)		$fc font_name		courier
	set_val vPlot($pid,font_style)		$fc font_style		medium
	set_val vPlot($pid,font_size)		$fc font_size		12

	set_val vPlot($pid,font_name_list)	$fc font_name_list "courier times helvetica lucida fixed"
	set_val vPlot($pid,font_style_list)	$fc font_style_list "medium bold"
	set_val vPlot($pid,font_size_list)	$fc font_size_list "8 9 10 12 14 18 20 24"

	set_val vPlot($pid,x_tics) 			$fc x_tics		5
	set_val vPlot($pid,y_tics) 			$fc y_tics		5
	set_val vPlot($pid,graph_label_text) 	$fc graph_label_text	""
	set_val vPlot($pid,graph_label_color) 	$fc graph_label_color	blue
	set_val vPlot($pid,graph_label_x_offset) 	$fc graph_label_x_offset 	0
	set_val vPlot($pid,graph_label_y_offset) 	$fc graph_label_y_offset -20
	set_val vPlot($pid,x_label_text)		$fc x_label_text 	""
	set_val vPlot($pid,y_label_text)		$fc y_label_text	""
	set_val vPlot($pid,x_label_color) 	$fc x_label_color	blue
	set_val vPlot($pid,y_label_color) 	$fc y_label_color	blue
	set_val vPlot($pid,x_label_x_offset) 	$fc x_label_x_offset 	0
	set_val vPlot($pid,x_label_y_offset) 	$fc x_label_y_offset 	30
	set_val vPlot($pid,y_label_x_offset) 	$fc y_label_x_offset 	10
	set_val vPlot($pid,y_label_y_offset) 	$fc y_label_y_offset 	20
	set_val vPlot($pid,x_border) 		$fc x_border		10 
	set_val vPlot($pid,y_border) 		$fc y_border		10 
	set_val vPlot($pid,x_format) 		$fc x_format 		"5.3g"
	set_val vPlot($pid,y_format) 		$fc y_format 		"5.3g"
	set_val vPlot($pid,margin_left) 		$fc margin_left 	80
    set_val vPlot($pid,margin_bottom)	$fc margin_bottom	70
    set_val vPlot($pid,margin_top)		$fc margin_top 		50
    set_val vPlot($pid,margin_right)		$fc margin_right 	40

	set_val vPlot($pid,bounded_color)	$fc bounded_color	black
	set_val vPlot($pid,axis_color) 		$fc axis_color 		blue
	set_val vPlot($pid,fit_color) 		$fc fit_color		red
	set_val vPlot($pid,fit_color_opt) 	$fc fit_color_opt	violet
	set_val vPlot($pid,fit_points) 		$fc fit_points		40
	set_val vPlot($pid,fit_line_width) 		$fc fit_line_width		2
	set_val vPlot($pid,data_symbol) 		$fc data_symbol		rect
	set_val vPlot($pid,data_color) 		$fc data_color		black
	set_val vPlot($pid,data_color_opt)	$fc data_color_opt	gray
	set_val vPlot($pid,data_label_color)	$fc data_label_color	blue
	set_val vPlot($pid,graph_label_text)	$fc graph_label_text 	""
	set_val vPlot($pid,data_label_text)		$fc data_label_text 	""
	set_val vPlot($pid,data_label_x_offset)	$fc data_label_x_offset		0
	set_val vPlot($pid,data_label_y_offset)	$fc data_label_y_offset		0
	set_val vPlot($pid,data_error_bars) 	$fc data_error_bars		no

    set_val vPlot($pid,col_max)			$fc col_max 		8
    set_val vPlot($pid,yes_color)			$fc yes_color 		red
    set_val vPlot($pid,no_color)			$fc no_color 		gray
}

#####################################################################
#
# update defaults file with user selection
#
proc write_plot_defaults { fpo } {
    global vPlot

	set pid 0

	puts $fpo "*canvas_geometry: $vPlot($pid,canvas_geometry)"
	puts $fpo "*canvas_bg_color: $vPlot($pid,canvas_bg_color)"
	puts $fpo "*highlightbackground: $vPlot($pid,canvas_bg_color)"
	puts $fpo "*margin_left: $vPlot($pid,margin_left)"
	puts $fpo "*margin_right: $vPlot($pid,margin_right)"
	puts $fpo "*margin_top: $vPlot($pid,margin_top)"
	puts $fpo "*margin_bottom: $vPlot($pid,margin_bottom)"
	puts $fpo "# "
	puts $fpo "*graph_label_text: $vPlot($pid,graph_label_text)"
	puts $fpo "*graph_label_x_offset: $vPlot($pid,graph_label_x_offset)"
	puts $fpo "*graph_label_y_offset: $vPlot($pid,graph_label_y_offset)"
	puts $fpo "*graph_label_color: $vPlot($pid,graph_label_color)"
	puts $fpo "# "
	puts $fpo "*x_label_text: $vPlot($pid,x_label_text)"
	puts $fpo "*y_label_text: $vPlot($pid,y_label_text)"
	puts $fpo "*x_label_x_offset: $vPlot($pid,x_label_x_offset)"
	puts $fpo "*x_label_y_offset: $vPlot($pid,x_label_y_offset)"
	puts $fpo "*y_label_x_offset: $vPlot($pid,y_label_x_offset)"
	puts $fpo "*y_label_y_offset: $vPlot($pid,y_label_y_offset)"
	puts $fpo "*x_label_color: $vPlot($pid,x_label_color)"
	puts $fpo "*y_label_color: $vPlot($pid,y_label_color)"
	puts $fpo "*x_tics: $vPlot($pid,x_tics)"
	puts $fpo "*y_tics: $vPlot($pid,y_tics)"
	puts $fpo "*x_format: $vPlot($pid,x_format)"
	puts $fpo "*y_format: $vPlot($pid,y_format)"
	puts $fpo "# "
	puts $fpo "*axis_color: $vPlot($pid,axis_color)"
	puts $fpo "*data_label_color: $vPlot($pid,data_label_color)"
	puts $fpo "*data_label_x_offset: $vPlot($pid,data_label_x_offset)"
	puts $fpo "*data_label_y_offset: $vPlot($pid,data_label_y_offset)"
	puts $fpo "*fit_color: $vPlot($pid,fit_color)"
	puts $fpo "*data_color: $vPlot($pid,data_color)"
	puts $fpo "*data_symbol: $vPlot($pid,data_symbol)"
	puts $fpo "*data_error_bars: $vPlot($pid,data_error_bars)"
	puts $fpo "*fit_points: $vPlot($pid,fit_points)"
	puts $fpo "*fit_line_width: $vPlot($pid,fit_line_width)"
}

###############################################################################
###############################################################################
###############################################################################

###############################################################################
#
# main routine for drawing one or more plots in one plot window
#
proc do_plot { pid iFile } {
	global vPlot panvar xcv

	# make sure we have read in some data
	if { $panvar(dataIndex) < 0 } {
		return
	}

	# make sure we have an empty plotting window  
	create_plot_win $pid
	plot_clear $vPlot($pid,canvas)

	# draw each plot in the specified plot window 
	for {set i 0} {$i < $vPlot($pid,pNumTot)} {incr i} {
		if {$vPlot($pid,$i,okFlag) == "TRUE"} {
			draw_one_plot $pid $i $iFile
		}
	}
	update
}

###############################################################################
#
# create plotting window 
#
proc create_plot_win { pid } {
	global vPlot xcv panvar

	# the window is already drawn on the screen
	set pWin .$xcv(win,plot,$pid)
	if [winfo exists $pWin] {
		return
	}

	# create the basic plotting window, a canvas within a toplevel frame
	if ![winfo exists $pWin] {
		toplevel $pWin
		set vPlot($pid,canvas) $pWin.canvas
		canvas $vPlot($pid,canvas) 
		pack $vPlot($pid,canvas) 
		get_plot_defaults $pid $vPlot($pid,canvas)
		wm geometry $pWin $vPlot($pid,canvas_geometry)
	} 

	# set the size of the plot window and background
	calibrate_geometry $pid
	$vPlot($pid,canvas) configure -background $vPlot($pid,canvas_bg_color)
	$vPlot($pid,canvas) configure -highlightbackground $vPlot($pid,canvas_bg_color)

	# create events for movable elements
	$vPlot($pid,canvas) bind movable <Button-1> {Mark %x %y %W}
	$vPlot($pid,canvas) bind movable <B1-Motion> {Drag %x %y %W}

	# the event we invoke when user resizes the plot window. 
	bind $vPlot($pid,canvas) <Configure> "plot_resize_event $pid"

	# update parameters so that if we have to draw a new plot window
	# it is drawn offset of this one. 
	set cVar [split $vPlot($pid,canvas_geometry) x+-]
	set new_x [expr [lindex $cVar 2] + 30]
	set new_y [expr [lindex $cVar 3] + 30]
	set next_pid [expr $pid + 1]
	set new_geo "[lindex $cVar 0]x[lindex $cVar 1]+$new_x+$new_y"
	set vPlot($next_pid,canvas_geometry) $new_geo
}

###############################################################################
#
# Delete all plots in the plot canvas 
#
proc plot_clear { aCanvas } {
	global vPlot

	# erase any old plot items 
	set items [$aCanvas find all]
	foreach item $items {
		$aCanvas delete $item
	}
}

###############################################################################
#
# draw one plot
#
proc draw_one_plot { pid pNum iFile } {
	global myData vPlot panvar

	draw_xScale $pid $pNum
	draw_yScale $pid $pNum
	draw_graph_title $pid $pNum

	set rc [check_parm_state]

	# draw fit functions first if they are shown 
	for {set j 0} {$j < $vPlot($pid,$pNum,dNumTot)} {incr j} {
		set loc [draw_fit $pid $pNum $j]
		if { [llength $loc] > 0 } {
			if { $vPlot($pid,$pNum,dNumCur) != $j } {
				draw_data_title $pid $pNum $j [lindex $loc 0] [lindex $loc 1]
			}
		}
	}

	# next draw bounding lines if data was constrained 
	if { $vPlot($pid,$pNum,bounded_flag) == 1 } {
		draw_bounded $pid $pNum $vPlot($pid,$pNum,bounded_left) -1
		draw_bounded $pid $pNum $vPlot($pid,$pNum,bounded_right) 1
	}

	# draw data at the end so that it doesn't get covered. 
	for {set j 0} {$j < $vPlot($pid,$pNum,dNumTot)} {incr j} {
		draw_data $pid $pNum $j $iFile
		if { $vPlot($pid,$pNum,dNumCur) == $j } {
			draw_data_title $pid $pNum $j 0 0 
		}
	}
}

###############################################################################
#
# draw data title 
# 
proc draw_data_title { pid pNum dNum xLoc yLoc } {
	global vPlot

	if { [string length $vPlot($pid,$pNum,$dNum,data_label_text)] <= 0 } {
		return
	}

	if { $xLoc > 0 } {
		set xstart [expr $xLoc + 10]
		set ystart $yLoc
	} else {
		set xstart [expr ($vPlot($pid,$pNum,pix_left) + $vPlot($pid,$pNum,pix_right)) / 2]
		set ystart [expr $vPlot($pid,$pNum,pix_top) - 16]
	}

	set xstart [expr $xstart + $vPlot($pid,$pNum,$dNum,data_label_x_offset)]
	set ystart [expr $ystart + $vPlot($pid,$pNum,$dNum,data_label_y_offset)]

	$vPlot($pid,canvas) create text $xstart $ystart \
		-font $vPlot($pid,$pNum,canvas_font) \
		-text $vPlot($pid,$pNum,$dNum,data_label_text) \
		-fill $vPlot($pid,$pNum,$dNum,data_label_color) -tag movable
}

###############################################################################
#
# draw graph title 
# 
proc draw_graph_title { pid pNum } {
	global vPlot

	if { [string length $vPlot($pid,$pNum,graph_label_text)] <= 0 } {
		return
	}

	set xstart [expr ($vPlot($pid,$pNum,pix_left) + $vPlot($pid,$pNum,pix_right)) / 2]
	set ystart [expr $vPlot($pid,$pNum,pix_top) - 16]

	set xstart [expr $xstart + $vPlot($pid,$pNum,graph_label_x_offset)]
	set ystart [expr $ystart + $vPlot($pid,$pNum,graph_label_y_offset)]

	$vPlot($pid,canvas) create text $xstart $ystart \
		-font $vPlot($pid,$pNum,canvas_font) \
		-text $vPlot($pid,$pNum,graph_label_text) \
		-fill $vPlot($pid,$pNum,graph_label_color) -tag movable
}

#
# draw x axis scale on graph 
# 
proc draw_xScale { pid pNum } {
	global vPlot

	$vPlot($pid,canvas) create line $vPlot($pid,$pNum,x_ref) $vPlot($pid,$pNum,y_origin) \
		[expr $vPlot($pid,$pNum,x_ref) + $vPlot($pid,$pNum,x_len)] \
		$vPlot($pid,$pNum,y_origin) -width 2 -fill $vPlot($pid,$pNum,axis_color) 

	set py [expr round($vPlot($pid,$pNum,y_origin) + $vPlot($pid,$pNum,x_border))]
	set tics [expr $vPlot($pid,$pNum,x_tics) - 1]
	set xInc [expr ($vPlot($pid,$pNum,x_max) - $vPlot($pid,$pNum,x_min)) / double($tics)]
	set pInc [expr double($vPlot($pid,$pNum,x_len)) / double($tics)]
	set xi $vPlot($pid,$pNum,x_min)
	set ticStart [expr $vPlot($pid,$pNum,y_origin) + 5]
	set ticEnd [expr $vPlot($pid,$pNum,y_origin) - 5]

	for {set i 0} {$i <= $tics} {incr i} {
		set px [expr round($vPlot($pid,$pNum,x_ref) + ($pInc * $i))]
		$vPlot($pid,canvas) create line $px $ticStart $px $ticEnd \
			-fill $vPlot($pid,$pNum,axis_color)
		set sVal [format "%$vPlot($pid,$pNum,x_format)" $xi]
		$vPlot($pid,canvas) create text $px $py -text $sVal \
			-fill $vPlot($pid,$pNum,axis_color) \
			-font $vPlot($pid,$pNum,canvas_font) -anchor n
		set xi [expr $xi + $xInc]
	}

	set px [expr round($vPlot($pid,$pNum,x_ref) + $vPlot($pid,$pNum,x_len) / 2) + $vPlot($pid,$pNum,x_label_x_offset)]
	set py [expr round($vPlot($pid,$pNum,y_ref) + $vPlot($pid,$pNum,x_label_y_offset))]

	$vPlot($pid,canvas) create text $px $py -text $vPlot($pid,$pNum,x_label_text) \
		-fill $vPlot($pid,$pNum,x_label_color) \
		-font $vPlot($pid,$pNum,canvas_font) \
		-anchor e -tag movable
}

#
# draw y axis scale on graph 
# 
proc draw_yScale { pid pNum } {
	global vPlot

	set lb [expr $vPlot($pid,$pNum,pix_top) + $vPlot($pid,$pNum,y_len)]
	$vPlot($pid,canvas) create line $vPlot($pid,$pNum,pix_left) $lb \
		$vPlot($pid,$pNum,pix_left) $vPlot($pid,$pNum,pix_top) -width 2 \
		-fill $vPlot($pid,$pNum,axis_color)

	set px [expr round($vPlot($pid,$pNum,pix_left) - $vPlot($pid,$pNum,y_border))]
	set tics [expr $vPlot($pid,$pNum,y_tics) - 1]
	set yInc [expr ($vPlot($pid,$pNum,y_max) - $vPlot($pid,$pNum,y_min)) / double($tics)]
	set pInc [expr double($vPlot($pid,$pNum,y_len)) / double($tics)]
	set yi $vPlot($pid,$pNum,y_min)
	set ticStart [expr $vPlot($pid,$pNum,pix_left) - 5]
	set ticEnd [expr $vPlot($pid,$pNum,pix_left) + 5]

	for {set i 0} {$i <= $tics} {incr i} {
		set py [expr round($vPlot($pid,$pNum,pix_top) + \
			$vPlot($pid,$pNum,y_len) - ($pInc * $i))]
		$vPlot($pid,canvas) create line $ticStart $py \
			$ticEnd $py -fill $vPlot($pid,$pNum,axis_color)
		set sVal [format "%$vPlot($pid,$pNum,y_format)" $yi]
		$vPlot($pid,canvas) create text $px $py -text $sVal \
			-fill $vPlot($pid,$pNum,axis_color) \
			-font $vPlot($pid,$pNum,canvas_font) -anchor e 
		set yi [expr $yi + $yInc]
	}

	set px [expr $px + $vPlot($pid,$pNum,y_label_x_offset)]
	set py [expr round($vPlot($pid,$pNum,pix_top) - $vPlot($pid,$pNum,y_label_y_offset))]
	$vPlot($pid,canvas) create text $px $py -text $vPlot($pid,$pNum,y_label_text) \
		-font $vPlot($pid,$pNum,canvas_font) \
		-fill $vPlot($pid,$pNum,y_label_color) \
		-tag movable
}

#
# draw data points on the screen 
# 
proc draw_data { pid pNum dNum iFile } {
	global vPlot myData

	set xRef $vPlot($pid,$pNum,x_ref)
	set xMin $vPlot($pid,$pNum,x_min)
	set yRef $vPlot($pid,$pNum,y_ref)
	set yMin $vPlot($pid,$pNum,y_min)
	set mix $vPlot($pid,$pNum,$dNum,mix)
	set data_symbol $vPlot($pid,$pNum,$dNum,data_symbol)
	set data_color $vPlot($pid,$pNum,$dNum,data_color)
	set data_error_bars $vPlot($pid,data_error_bars)

	# rmse data is in a fixed spot for now
	if { $pid == 2 } {
		set iFile $pid
	}

# if data has 2 columns or we are plotting error data from gridfit
	if { $myData($iFile,$mix,nCol) == 2 } {
		for {set j 0} {$j < $myData($iFile,$mix,iDataNum)} {incr j} {
			set px [expr round($xRef + ($myData($iFile,$mix,$j,xd) - $xMin) * \
				$vPlot($pid,$pNum,x_ratio))]
			set py [expr round($yRef - ($myData($iFile,$mix,$j,yd) - $yMin) * \
				$vPlot($pid,$pNum,y_ratio))]
			draw_one_data $pid $data_symbol $px $py $data_color

			if {$myData($iFile,$mix,$j,xlb) != $myData($iFile,$mix,$j,xub) || 
			    $myData($iFile,$mix,$j,ylb) != $myData($iFile,$mix,$j,yub)} {
				set xlb [expr round($xRef + ($myData($iFile,$mix,$j,xlb) \
					- $xMin) * $vPlot($pid,$pNum,x_ratio))]
				set xub [expr round($xRef + ($myData($iFile,$mix,$j,xub) \
					- $xMin) * $vPlot($pid,$pNum,x_ratio))]
				set ylb [expr round($yRef - ($myData($iFile,$mix,$j,ylb) \
					- $yMin) * $vPlot($pid,$pNum,y_ratio))]
				set yub [expr round($yRef - ($myData($iFile,$mix,$j,yub) \
					- $yMin) * $vPlot($pid,$pNum,y_ratio))]
				draw_one_errBar $pid $px $py $xlb $xub $ylb $yub $data_color $data_error_bars
			}
		}
		return "$px $py"
	}
	if {$myData(0,$mix,nCol) == 3} {
		for {set j 0} {$j < $myData(0,$mix,iDataNum)} {incr j} {
			set x0 [expr $myData($iFile,$mix,$j,xd) / $myData($iFile,$mix,$j,td) ]
			set px [expr round($xRef + ($x0 - $xMin) * $vPlot($pid,$pNum,x_ratio))]
			set py [expr round($yRef - ($myData($iFile,$mix,$j,yd) - $yMin) * \
				$vPlot($pid,$pNum,y_ratio))]
			draw_one_data $pid $data_symbol $px $py $data_color

			if {$myData($iFile,$mix,$j,xlb) != $myData($iFile,$mix,$j,xub) || 
			    $myData($iFile,$mix,$j,ylb) != $myData($iFile,$mix,$j,yub)} {
				set xlb [expr round($xRef + ($myData($iFile,$mix,$j,xlb) \
					- $xMin) * $vPlot($pid,$pNum,x_ratio))]
				set xub [expr round($xRef + ($myData($iFile,$mix,$j,xub) \
					- $xMin) * $vPlot($pid,$pNum,x_ratio))]
				set ylb [expr round($yRef - ($myData($iFile,$mix,$j,ylb) \
					- $yMin) * $vPlot($pid,$pNum,y_ratio))]
				set yub [expr round($yRef - ($myData($iFile,$mix,$j,yub) \
					- $yMin) * $vPlot($pid,$pNum,y_ratio))]
				draw_one_errBar $pid $px $py $xlb $xub $ylb $yub $data_color $data_error_bars
			}
		}
	}
	return "$px $py"
}

#
# draw one data symbol on the plotting canvas 
# 
proc draw_one_data {pid symbol px py color} {
	global panvar vPlot

	if {$symbol == "diamond"} {
		$vPlot($pid,canvas) create line [expr $px-4] [expr $py] \
			[expr $px] [expr $py+4] -fill $color -width 2
		$vPlot($pid,canvas) create line [expr $px] [expr $py+4] \
			[expr $px+4] [expr $py] -fill $color -width 2
		$vPlot($pid,canvas) create line [expr $px+4] [expr $py] \
			[expr $px] [expr $py-4] -fill $color -width 2
		$vPlot($pid,canvas) create line [expr $px] [expr $py-4] \
			[expr $px-4] [expr $py] -fill $color -width 2
		return
	}
	if {$symbol == "cross"} {
		$vPlot($pid,canvas) create line [expr $px-4] [expr $py] \
			[expr $px+4] [expr $py] -fill $color -width 2
		$vPlot($pid,canvas) create line [expr $px] [expr $py+4] \
			[expr $px] [expr $py-4] -fill $color -width 2
		return
	}

	# rectangle and ovals are special
	$vPlot($pid,canvas) create $symbol [expr $px-3] [expr $py-3] \
		[expr $px+3] [expr $py+3] -outline $color -width 2
}

#
# draw error bounds on data 
# 
proc draw_one_errBar {pid px py xlb xub ylb yub color ok} {
	global panvar vPlot

	if { $ok == "no" } { return }

	if { $ylb != $yub } {
		set lb [expr $px - 3]
		set ub [expr $px + 3]
		$vPlot($pid,canvas) create line $px $ylb $px $yub -width 1 -fill $color
		$vPlot($pid,canvas) create line $lb $ylb $ub $ylb -width 1 -fill $color
		$vPlot($pid,canvas) create line $lb $yub $ub $yub -width 1 -fill $color
	}

	if { $xlb != $xub } {
		set lb [expr $py - 3]
		set ub [expr $py + 3]
		$vPlot($pid,canvas) create line $xlb $py $xub $py -width 1 -fill $color
		$vPlot($pid,canvas) create line $xlb $lb $xlb $ub -width 1 -fill $color
		$vPlot($pid,canvas) create line $xub $lb $xub $ub -width 1 -fill $color
	}
}

#
# draw bounds on data 
# 
proc draw_bounded {pid pNum xVal inc} {
	global panvar vPlot myData

	set xMin $vPlot($pid,$pNum,x_min)
	set lb [expr $vPlot($pid,$pNum,pix_top) + $vPlot($pid,$pNum,y_len)]
	set ub $vPlot($pid,$pNum,pix_top)
	set px [expr $vPlot($pid,$pNum,x_ref) + ($xVal - $xMin) * $vPlot($pid,$pNum,x_ratio)]
	$vPlot($pid,canvas) create line $px $lb $px $ub -width 2 \
		-fill $vPlot($pid,$pNum,bounded_color) 
}

#
# draw fitting function for a dataset. 
# 
proc draw_fit { pid pNum dNum } {
	global vPlot

	if ![info exists vPlot($pid,$pNum,$dNum,fitx,0)] { return }
	if { $pid == 2 } { return }

	set xRef $vPlot($pid,$pNum,x_ref)
	set yRef $vPlot($pid,$pNum,y_ref)
	set xMin $vPlot($pid,$pNum,x_min)
	set yMin $vPlot($pid,$pNum,y_min)
	set yMax $vPlot($pid,$pNum,y_max)
	set fit_color $vPlot($pid,$pNum,$dNum,fit_color)

	for {set j 0} {$j <= $vPlot($pid,$pNum,$dNum,fit_points)} {incr j} {
		set px2 [expr round($xRef + \
			($vPlot($pid,$pNum,$dNum,fitx,$j) - $xMin) * $vPlot($pid,$pNum,x_ratio))]
		set py2 [expr round($yRef - \
			($vPlot($pid,$pNum,$dNum,fity,$j) - $yMin) * $vPlot($pid,$pNum,y_ratio))]
		if { $vPlot($pid,$pNum,$dNum,valid,$j) == 1 && $j > 0 } {

			# if multiple plots, make sure we only draw in the current plot
			if { ($pid < 3) || (($vPlot($pid,$pNum,$dNum,fity,$j) >= $yMin)
			&& ($vPlot($pid,$pNum,$dNum,fity,$j) <= $yMax)) } {

				$vPlot($pid,canvas) create line $px1 $py1 $px2 $py2 -fill \
					$fit_color -width $vPlot($pid,$pNum,$dNum,fit_line_width) 

			}
		}
		set px1 $px2
		set py1 $py2
	}
	return "$px2 $py2"
}

###############################################################################
#
# Allows user to drag elements of the plot (eg, titles) to more
# appropriate places
#
proc Mark {x y w } {
	global state
	set state($w,obj) [$w find closest $x $y]
	set state($w,x) $x
	set state($w,y) $y
}
proc Drag {x y w} {
	global state
	set dx [expr $x - $state($w,x)]
	set dy [expr $y - $state($w,y)]
	$w move $state($w,obj) $dx $dy
	set state($w,x) $x
	set state($w,y) $y
}
